/*-
 * Copyright (c) 2005 Masashi Osakabe
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#ifndef JAVA_LANG_CLASS_H
#define JAVA_LANG_CLASS_H

#include <dlfcn.h>
#include <iostream>
#include <boost/shared_ptr.hpp>

#include <sl/class_loader.hpp>
#include <sl/java/lang/Object.h>
#include <sl/java/lang/InstantiationException.h>

namespace sl {
namespace java {
namespace lang {

class Class {
public :
    Class()
    { }

    Class(sl::class_loader* loader)
        : _loader(loader)
    { }

    ~Class()
    { }

    template <class ClassType>
    boost::shared_ptr<ClassType> newInstance()
        throw (InstantiationException)
    {
        ClassType* ptr;
        try {
            ptr = _loader->template instance<ClassType>();
        } catch (std::runtime_error& e) {
            throw InstantiationException(e.what());
        }
        return boost::shared_ptr<ClassType>(ptr);
    }

private :
    sl::class_loader* _loader;
};

} // namespace lang
} // namespace java
} // namespace sl

#endif // JAVA_LANG_CLASS_H
