/*-
 * Copyright (c) 2005 Masashi Osakabe
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#ifndef JAVA_UTIL_ENUMERATION_H
#define JAVA_UTIL_ENUMERATION_H

#include <vector>
#include <sl/java/lang/Object.h>

namespace sl {
namespace java {
namespace util {

class Enumeration {
public :
    virtual ~Enumeration() { }

    Enumeration(std::vector<lang::Object>& objs) : _objs(objs), _pos(0) { }

    virtual Enumeration& operator=(const Enumeration &rhs)
    {
        _objs = rhs._objs;
        _pos = rhs._pos;
        return *this;
    }

    virtual bool hasMoreElements() const
    {
        return _objs.begin() + _pos < _objs.end();
    }

    virtual lang::Object nextElement()
    {
        return _objs[_pos++];
    }

    virtual operator std::vector<lang::Object>()
    {
        return _objs;
    }

protected :
    Enumeration() { }

    std::vector<lang::Object> _objs;
    size_t _pos;
};

} // namespace util
} // namespace java
} // namespace sl

#endif // JAVA_UTIL_ENUMERATION_H
