/*-
 * Copyright (c) 2005 Masashi Osakabe
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: end_response_packet.cpp,v 1.3 2006/10/28 16:05:57 cvsuser Exp $
 */

#include <sl/net/ajp13/ajp13_error.h>
#include <sl/net/ajp13/end_response_packet.h>
#include <sl/net/ajp13/function.h>
using namespace sl::net::ajp13;


Byte end_response_packet::prefix_code() const
{
	return prefix_code::END_RESPONSE;
}


packet &end_response_packet::operator=(const ByteArray &array)
	throw(ajp13_error)
{
	const Byte *p = array;

	if (*p != this->prefix_code())
		throw std::invalid_argument("invalid prefix code");

	++p;	// skip prefix_code
	_reuse = get_boolean(p);

	return *this;
}


end_response_packet::operator ByteArray() const
{
	ByteArray array;
	array += prefix_code();
	array += _reuse;

	return array;
}


Integer end_response_packet::size() const
{
	Integer s = 0;
	s += sizeof(Byte);		// prefix_code
	s += sizeof(Boolean);	// reuse

	return s;
}

void end_response_packet::reuse(Boolean r)
{
	_reuse = r;
}

Boolean end_response_packet::reuse() const
{
	return _reuse;
}

void end_response_packet::clear()
{ }
