/*-
 * Copyright (c) 2005 masashi osakabe
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#ifndef SJ_ENGINE_BODY_CONTENT_IMPL_H
#define SJ_ENGINE_BODY_CONTENT_IMPL_H

#include <servlet/jsp/tagext/BodyContent.h>

namespace sj {

class BodyContentImpl : public servlet::jsp::tagext::BodyContent {
public :
    BodyContentImpl(servlet::jsp::JspWriter& writer)
        : servlet::jsp::tagext::BodyContent(writer), _writer(&writer)
    { }

    void write(const std::vector<char>& s, int off, int len)
    {
        _writer->write(s, off, len);
    }

    virtual void clearBody()
    {
        _buffer.clear();
    }

    virtual void flush()
    {
        _writer->write(_buffer);
    }

    virtual servlet::jsp::JspWriter& getEnclosingWriter()
    {
        return *_writer;
    }

    virtual sl::java::io::Reader& getReader()
    {
    }

    virtual std::string getString()
    {
        return _buffer;
    }

    virtual void writeOut(std::ostream &out)
    {
        out << _buffer;
    }

private :
    servlet::jsp::JspWriter* _writer;
    std::string _buffer;
};

} // namespace sj

#endif // SJ_ENGINE_BODY_CONTENT_IMPL_H
