/*-
 * Copyright (c) 2005 masashi osakabe
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#ifndef SHIBAINU_SERVER_CONFIG_H
#define SHIBAINU_SERVER_CONFIG_H

#include <map>
#include <string>
#include <vector>
#include <sl/xml/xml_tag.h>

#include <si/interface/Config.h>

namespace si {
namespace core {

class ServerConfig : public interface::Config {
public :
    static const std::string    SERVER_XML;

    /** Constractor. */
    ServerConfig();

    /** Constructor. */
    ServerConfig(const std::string& home);

    void parseServerXML(const std::string& home);

private :

    interface::Config
    parseAttribute(sl::xml::xml_tag& tag, const std::string& name);

    interface::Config parseServiceTag(sl::xml::xml_tag& tag);

    interface::Config parseEngineTag(sl::xml::xml_tag& tag);

    interface::Config parseLoggerTag(sl::xml::xml_tag& tag);

    interface::Config parseRealmTag(sl::xml::xml_tag& tag);

    std::vector<interface::Config>
    parseConnectorTag(sl::xml::xml_tag& tag);

    std::vector<interface::Config>
    parseHostTag(sl::xml::xml_tag& tag);

    std::vector<interface::Config>
    parseContextTag(sl::xml::xml_tag& tag);

    std::vector<interface::Config>
    parseValveTag(sl::xml::xml_tag& tag);
};

} // namespace core
} // namespace si

#endif // SHIBAINU_SERVER_CONFIG_H
