/*-
 * Copyright (c) 2005 masashi osakabe
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: DispatcherExample.cpp,v 1.2 2008/03/11 15:54:31 cvsuser Exp $
 */

#include <sl/class_loader.hpp>
#include <sl/java.hpp>
#include <servlet/servlet_headers.h>
using namespace sl::java::lang;
using namespace sl::java::io;
using namespace sl::java::util;
using namespace servlet;
using namespace servlet::http;


class DispatcherExample : public HttpServlet {
public :

    void doGet(HttpServletRequest &req, HttpServletResponse &res)
        throw(ServletException)
    {
        res.setContentType("text/html;charset=EUC-JP");

		PrintWriter out = res.getWriter();

        Enumeration p = req.getParameterNames();
        while (p.hasMoreElements()) {
            try {
                std::string name = sl::object_cast<std::string>(p.nextElement());
				if (name == "value1") {
					int i = atoi(req.getParameter(name).c_str());
					req.setAttribute("value1", i);
				}

				if (name == "value2") {
					req.setAttribute("value2", req.getParameter(name));
				}
            } catch (std::bad_cast& e) {
                std::cerr << "DispatcherExample:" << e.what() << std::endl;
            }
        }

		std::auto_ptr<RequestDispatcher> rd =
					getServletContext().getRequestDispatcher("/TestPage.jsp");
		rd->forward(req, res);
    }

    void doPost(HttpServletRequest &req, HttpServletResponse &res)
        throw(ServletException)
    {
		doGet(req, res);
	}
};

SL_REGIST_LOADABLE_CLASS(DispatcherExample)
